/*
 * *****************************************************************************************
 *
 * 		Copyright (C) 2014-2018 Gowin Semiconductor Technology Co.,Ltd.
 * 		
 * @file			main.c
 * @author		Embedded Development Team
 * @version		V1.0.0
 * @date			2018-5-1 09:00:00
 * @brief			main function
 ******************************************************************************************
 */
#include "gw1ns2c.h"
#include "includes.h"

#define SW_STR_NAME							"GW1NS_2C_UCOSIII_V3.03"	//software name
#define SW_STR_EDITION					"V1.0.0.181029a"					//software version
#define SW_STR_AUTHOR						"GOWIN"										//author


#define LOGO_PRINT_ON

#ifdef LOGO_PRINT_ON
#include "logo.h"
#endif	//LOGO_PRINT_ON


//There are file system internal tasks in UCOSIII, user can't use theirs
//Priority-0: Interrupt service manage task OS_IntQTask()
//Priority-1: Clock tick task OS_TickTask()
//Priority-2: Timer task OS_TmrTask()
//Priority OS_CFG_PRIO_MAX-2: stat task OS_StatTask()
//Priority OS_CFG_PRIO_MAX - 1: Idle task OS_IdleTask()

#define LED0_TASK_PRIO			3
#define LED0_STK_SIZE 			48
OS_TCB Led0TaskTCB;
CPU_STK LED0_TASK_STK[LED0_STK_SIZE];
static void led0_task(void *p_arg);

#define LED1_TASK_PRIO			4
#define LED1_STK_SIZE 			48
OS_TCB Led1TaskTCB;
CPU_STK LED1_TASK_STK[LED1_STK_SIZE];
static void led1_task(void *p_arg);


volatile int led0_task_flag = 0;
volatile int led1_task_flag = 0;


static void printf_str(const char *str)
{
	UART_SendString(UART0, (char *)str);
}

static void stars_print(uint8_t n)
{
	while (n--)
	{
		printf_str("*");
	}
}

static void sw_edition_print(void)
{
	printf_str("\r\n");
	stars_print(48);
	printf_str("\r\n");
	//printf_str("************************************************\r\n");
	printf_str("Name:     "SW_STR_NAME"\r\n"
           "Edition:  "SW_STR_EDITION"\r\n"
		   "Compiled: "__DATE__", "__TIME__"\r\n"
			"Author:   "SW_STR_AUTHOR"\r\n");
	//printf_str("************************************************\r\n");
	stars_print(48);
	printf_str("\r\n\r\n");
}

static void gpio_user_init(void)
{
	GPIO0->OUTENSET = 0xFFFFFFFF;
	GPIO_SetBit(GPIO0, GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3);
}

static void uart_user_init(uint32_t baudrate)
{
	UART_InitTypeDef UART_InitStruct;

	UART_InitStruct.UART_Mode.UARTMode_Tx = ENABLE;
	UART_InitStruct.UART_Mode.UARTMode_Rx = ENABLE;
	UART_InitStruct.UART_Int.UARTInt_Tx = DISABLE;
	UART_InitStruct.UART_Int.UARTInt_Rx = DISABLE;
	UART_InitStruct.UART_Ovr.UARTOvr_Tx = DISABLE;
	UART_InitStruct.UART_Ovr.UARTOvr_Rx = DISABLE;
	UART_InitStruct.UART_Hstm = DISABLE;
	UART_InitStruct.UART_BaudRate = baudrate;

	UART_Init(UART0,&UART_InitStruct);
}

static void sys_tick_init(void)
{
	uint32_t temp;
	
	//24-bit register, max value is 16777215
	//when systemCoreClock is 25MHz, this is 671ms
	//Set reload register's value
	temp = (1000 / OSCfg_TickRate_Hz) * (SystemCoreClock / 1000) - 1;
	SysTick->LOAD = temp;
	
	SysTick->VAL = temp;//Reset current counter's value
	
	//select clock source, enable interrupt, enable counter
	SysTick->CTRL = SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk; 
}

void SysTick_Handler(void)
{
	if (OSRunning == OS_STATE_OS_RUNNING)
	{
		OSIntEnter();
		OSTimeTick();
		OSIntExit();
	}
}

static void led0_task(void *p_arg)
{
	OS_ERR err;
	//p_arg = p_arg;

	while(1)
	{
		if (0 == led0_task_flag)
		{
			GPIO_ResetBit(GPIO0, GPIO_Pin_0);
		}
		else
		{
			GPIO_SetBit(GPIO0, GPIO_Pin_0);
		}
		
		printf_str("0.task0\r\n");
		led0_task_flag = !led0_task_flag;
		OSTimeDlyHMSM(0, 0, 0, 250, OS_OPT_TIME_HMSM_STRICT, &err);
	}
}

static void led1_task(void *p_arg)
{
	OS_ERR err;
	//p_arg = p_arg;

	while(1)
	{
		if (0 == led1_task_flag)
		{
			GPIO_ResetBit(GPIO0, GPIO_Pin_1);
		}
		else
		{
			GPIO_SetBit(GPIO0, GPIO_Pin_1);
		}
		
		printf_str("1.task1\r\n\r\n");
		led1_task_flag = !led1_task_flag;
		OSTimeDlyHMSM(0, 0, 0, 1000, OS_OPT_TIME_HMSM_STRICT, &err);
	}
}

int main(void)
{
	OS_ERR err;
	CPU_SR_ALLOC();

	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_3);

	uart_user_init(115200);
	gpio_user_init();
	
	sw_edition_print();
	
#ifdef LOGO_PRINT_ON
	printf_str(LOGO);
#endif	//LOGO_PRINT_ON

	sys_tick_init();
	OSInit(&err);
	CPU_Init();
	
	OS_CRITICAL_ENTER();
	
	OSTaskCreate((OS_TCB *		)&Led0TaskTCB,		
				 (CPU_CHAR *	)"led0 task", 		
                 (OS_TASK_PTR	)led0_task, 			
                 (void *		)0,					
                 (OS_PRIO		)LED0_TASK_PRIO,     
                 (CPU_STK *		)&LED0_TASK_STK[0],	
                 (CPU_STK_SIZE	)(LED0_STK_SIZE / 10),	
                 (CPU_STK_SIZE	)LED0_STK_SIZE,		
                 (OS_MSG_QTY	)0,					
                 (OS_TICK		)0,					
                 (void *		)0,					
                 (OS_OPT		)OS_OPT_TASK_STK_CHK|OS_OPT_TASK_STK_CLR,
                 (OS_ERR *		)&err);				
				 
	OSTaskCreate((OS_TCB *		)&Led1TaskTCB,		
				 (CPU_CHAR *	)"led1 task", 		
                 (OS_TASK_PTR	)led1_task, 			
                 (void *		)0,					
                 (OS_PRIO		)LED1_TASK_PRIO,     	
                 (CPU_STK *		)&LED1_TASK_STK[0],	
                 (CPU_STK_SIZE	)(LED1_STK_SIZE / 10),	
                 (CPU_STK_SIZE	)LED1_STK_SIZE,		
                 (OS_MSG_QTY	)0,					
                 (OS_TICK		)0,					
                 (void *		)0,				
                 (OS_OPT		)OS_OPT_TASK_STK_CHK|OS_OPT_TASK_STK_CLR, 
                 (OS_ERR *		)&err);
	
	OS_CRITICAL_EXIT();
	
	OSStart(&err);
	
	while(1);
}
